#!/usr/bin/env python

__author__ = "Bryce Cousins (bfc5288@psu.edu)"
__description__ = "tools to receive and parse GCN circulars"


import argparse
import json

from .io import Stream


def print_gcn(gcn_dict, json_dump=False):
    """Parse a gcn dictionary and print to stdout.

    Args:
      gcn_dict:  the dictionary object containing a GCN-formatted message

    Returns:
      None

    """
    if json_dump:
        print(json.dumps(gcn_dict))
    else:
        gcn_headers = gcn_dict["header"]
        gcn_body = gcn_dict["body"]

        for key in gcn_headers:
            line = key.upper() + ":\t" + gcn_headers[key]
            print(line)

        for line in gcn_body.splitlines():
            print(line)


# ------------------------------------------------
# -- main


def _add_parser_args(parser):
    parser.add_argument(
        "url",
        metavar="URL",
        help="Sets the broker URL (kafka://host[:port]/topic) from which to receive GCNs.",
    )

    # configuration options
    config = parser.add_mutually_exclusive_group()
    config.add_argument(
        "-F", "--config-file", help="Set client configuration from file.",
    )
    config.add_argument(
        "-X",
        "--config",
        action="append",
        help="Set client configuration via prop=val. Can be specified multiple times.",
    )

    # consumer options
    parser.add_argument(
        "-j", "--json", help="Request gcn output as raw json", action="store_true",
    )
    parser.add_argument(
        "-e",
        "--earliest",
        help="Request to stream from the earliest available Kafka offset",
        action="store_true",
    )
    parser.add_argument(
        "-t",
        "--timeout",
        type=float,
        help="Specifies the time (in seconds) to wait for new messages.",
    )


def _main(args=None):
    """Receive and parse GCN circulars.

    """
    if not args:
        parser = argparse.ArgumentParser()
        _add_parser_args(parser)
        args = parser.parse_args()

    # load config if specified
    if args.config_file:
        config = args.config_file
    elif args.config:
        config = {opt[0]: opt[1] for opt in (kv.split("=") for kv in args.config)}
    else:
        config = None

    # load consumer options

    # defaults:
    start_offset = "latest"
    timeout = 10
    json_dump = False

    if args.json:
        json_dump = True
    if args.earliest:
        start_offset = "earliest"
    if args.timeout:
        timeout = args.timeout

    # read from topic

    # assume json format for the gcn
    gcn_format = "json"

    stream = Stream(format=gcn_format, config=config, start_at=start_offset)
    with stream.open(args.url, "r") as s:
        for _, gcn_dict in s(timeout=timeout):
            print_gcn(gcn_dict, json_dump)



#!/usr/bin/env python

__author__ = "Patrick Godwin (patrick.godwin@psu.edu)"
__description__ = "global configuration for pytest suite"


import io

import pytest


# example GCN circular from https://gcn.gsfc.nasa.gov/gcn3_circulars.html

GCN_TITLE = "GCN GRB OBSERVATION REPORT"
GCN_NUMBER = "40"
GCN_SUBJECT = "GRB980329 VLA observations"
GCN_DATE = "98/04/03 07:10:15 GMT"
GCN_FROM = "Greg Taylor at NRAO"
GCN_BODY = """
G.B. Taylor, D.A. Frail (NRAO), S.R. Kulkarni (Caltech), and
the BeppoSAX GRB team report:

We have observed the field containing the proposed x-ray counterpart
1SAX J0702.6+3850 of GRB 980329 (IAUC 6854) with the VLA at 8.4 GHz
on UT 1998 Mar 30.2, April 1.1, and April 2.1.  Observations on April
1.1 detected a radio source VLA J0702+3850 within the 1 arcminute
error circle of 1SAX J0702.6+3850.  The coordinates of
VLA J0702+3850 are: ra = 07h02m38.02170s dec = 38d50'44.0170" (equinox
J2000) with an uncertainty of 0.05 arcsec in each coordinate.  The
size of this radio source is less than 0.25 arcsec.  The density of
sources on the sky stronger than 250 microJy at this frequency is
0.0145 arcmin**-2.

The flux density measurements of VLA J0702+3850 are as follows:

Date(UT)   8.4 GHz Flux Density
--------   ----------------------
Mar 30.2   166 +/- 50 microJy
Apr  1.1   248 +/- 16    "
Apr  2.1    65 +/- 25    "

where the uncertainty in the measurement reflects the 1 sigma rms
noise in the image.  These measurements clearly demonstrate that
the radio source is variable on timescales of less than 1 day.
This rapid variability is similar to that observed in the
radio afterglow from GRB 970508.  We propose VLA J0702+3850
is the radio afterglow from GRB 980329.

Additional radio observations are in progress.
"""

GCN_CIRCULAR = f"""\
TITLE:   {GCN_TITLE}
NUMBER:  {GCN_NUMBER}
SUBJECT: {GCN_SUBJECT}
DATE:    {GCN_DATE}
FROM:    {GCN_FROM}

{GCN_BODY}\
"""

VOEVENT_XML = """\
<?xml version='1.0' encoding='UTF-8'?>
<voe:VOEvent xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:voe="http://www.ivoa.net/xml/VOEvent/v2.0" xsi:schemaLocation="http://www.ivoa.net/xml/VOEvent/v2.0 http://www.ivoa.net/xml/VOEvent/VOEvent-v2.0.xsd" version="2.0" role="observation" ivorn="ivo://gwnet/LVC#S200302c-1-Preliminary">
  <Who>
    <Date>2020-03-02T02:00:09</Date>
    <Author>
      <contactName>LIGO Scientific Collaboration and Virgo Collaboration</contactName>
    </Author>
  </Who>
  <What>
    <Param dataType="int" name="Packet_Type" value="150">
      <Description>The Notice Type number is assigned/used within GCN, eg type=150 is an LVC_PRELIMINARY notice</Description>
    </Param>
    <Param dataType="int" name="internal" value="0">
      <Description>Indicates whether this event should be distributed to LSC/Virgo members only</Description>
    </Param>
    <Param dataType="int" name="Pkt_Ser_Num" value="1">
      <Description>A number that increments by 1 each time a new revision is issued for this event</Description>
    </Param>
    <Param dataType="string" name="GraceID" ucd="meta.id" value="S200302c">
      <Description>Identifier in GraceDB</Description>
    </Param>
    <Param dataType="string" name="AlertType" ucd="meta.version" value="Preliminary">
      <Description>VOEvent alert type</Description>
    </Param>
    <Param dataType="int" name="HardwareInj" ucd="meta.number" value="0">
      <Description>Indicates that this event is a hardware injection if 1, no if 0</Description>
    </Param>
    <Param dataType="int" name="OpenAlert" ucd="meta.number" value="1">
      <Description>Indicates that this event is an open alert if 1, no if 0</Description>
    </Param>
    <Param dataType="string" name="EventPage" ucd="meta.ref.url" value="https://gracedb.ligo.org/superevents/S200302c/view/">
      <Description>Web page for evolving status of this GW candidate</Description>
    </Param>
    <Param dataType="string" name="Instruments" ucd="meta.code" value="H1,V1">
      <Description>List of instruments used in analysis to identify this event</Description>
    </Param>
    <Param dataType="float" name="FAR" ucd="arith.rate;stat.falsealarm" unit="Hz" value="9.349090689402942e-09">
      <Description>False alarm rate for GW candidates with this strength or greater</Description>
    </Param>
    <Param dataType="string" name="Group" ucd="meta.code" value="CBC">
      <Description>Data analysis working group</Description>
    </Param>
    <Param dataType="string" name="Pipeline" ucd="meta.code" value="gstlal">
      <Description>Low-latency data analysis pipeline</Description>
    </Param>
    <Param dataType="string" name="Search" ucd="meta.code" value="AllSky">
      <Description>Specific low-latency search</Description>
    </Param>
    <Group name="GW_SKYMAP" type="GW_SKYMAP">
      <Param dataType="string" name="skymap_fits" ucd="meta.ref.url" value="https://gracedb.ligo.org/api/superevents/S200302c/files/bayestar.fits.gz,0">
        <Description>Sky Map FITS</Description>
      </Param>
    </Group>
    <Group name="Classification" type="Classification">
      <Param dataType="float" name="BNS" ucd="stat.probability" value="0.0">
        <Description>Probability that the source is a binary neutron star merger (both objects lighter than 3 solar masses)</Description>
      </Param>
      <Param dataType="float" name="NSBH" ucd="stat.probability" value="0.0">
        <Description>Probability that the source is a neutron star-black hole merger (primary heavier than 5 solar masses, secondary lighter than 3 solar masses)</Description>
      </Param>
      <Param dataType="float" name="BBH" ucd="stat.probability" value="0.8895532192171397">
        <Description>Probability that the source is a binary black hole merger (both objects heavier than 5 solar masses)</Description>
      </Param>
      <Param dataType="float" name="MassGap" ucd="stat.probability" value="0.0">
        <Description>Probability that the source has at least one object between 3 and 5 solar masses</Description>
      </Param>
      <Param dataType="float" name="Terrestrial" ucd="stat.probability" value="0.11044678078286028">
        <Description>Probability that the source is terrestrial (i.e., a background noise fluctuation or a glitch)</Description>
      </Param>
      <Description>Source classification: binary neutron star (BNS), neutron star-black hole (NSBH), binary black hole (BBH), MassGap, or terrestrial (noise)</Description>
    </Group>
    <Group name="Properties" type="Properties">
      <Param dataType="float" name="HasNS" ucd="stat.probability" value="0.0">
        <Description>Probability that at least one object in the binary has a mass that is less than 3 solar masses</Description>
      </Param>
      <Param dataType="float" name="HasRemnant" ucd="stat.probability" value="0.0">
        <Description>Probability that a nonzero mass was ejected outside the central remnant object</Description>
      </Param>
      <Description>Qualitative properties of the source, conditioned on the assumption that the signal is an astrophysical compact binary merger</Description>
    </Group>
  </What>
  <WhereWhen>
    <ObsDataLocation>
      <ObservatoryLocation id="LIGO Virgo"/>
      <ObservationLocation>
        <AstroCoordSystem id="UTC-FK5-GEO"/>
        <AstroCoords coord_system_id="UTC-FK5-GEO">
          <Time unit="s">
            <TimeInstant>
              <ISOTime>2020-03-02T01:58:11.519119</ISOTime>
            </TimeInstant>
          </Time>
        </AstroCoords>
      </ObservationLocation>
    </ObsDataLocation>
  </WhereWhen>
  <Description>Report of a candidate gravitational wave event</Description>
  <How>
    <Description>Candidate gravitational wave event identified by low-latency analysis</Description>
    <Description>V1: Virgo 3 km gravitational wave detector</Description>
    <Description>H1: LIGO Hanford 4 km gravitational wave detector</Description>
  </How>
</voe:VOEvent>\
"""


@pytest.fixture(scope="session")
def circular_text():
    return GCN_CIRCULAR


@pytest.fixture(scope="session")
def circular_msg():
    return {
        "header": {
            "title": GCN_TITLE,
            "number": GCN_NUMBER,
            "subject": GCN_SUBJECT,
            "date": GCN_DATE,
            "from": GCN_FROM,
        },
        "body": GCN_BODY,
    }


@pytest.fixture(scope="session")
def voevent_fileobj():
    return io.BytesIO(VOEVENT_XML.encode())


@pytest.fixture(scope="session")
def voevent_text():
    return VOEVENT_XML



#!/usr/bin/env python

__author__ = "Patrick Godwin (patrick.godwin@psu.edu)"
__description__ = "a module for i/o utilities"


from genesis import streaming


class Stream(object):
    """Defines an event stream.

    Sets up defaults used within the genesis client, so that when a
    stream connection is opened, it will use defaults specified here.

    Args:
      format: the message serialization format
      start_at: the message offset to start at
      config: librdkafka style options, either a `dict` or a file path

    """

    def __init__(self, format=None, start_at=None, config=None):
        self._options = {}
        if format is not None:
            self._options["format"] = format
        if start_at is not None:
            self._options["start_at"] = start_at
        if config is not None:
            self._options["config"] = config

    def open(self, *args, **kwargs):
        """Opens a connection to an event stream.

        Args:
          broker_url: sets the broker URL to connect to

        Kwargs:
          mode: read ('r') or write ('w') from the stream
          format: the message serialization format
          start_at: the message offset to start at
          config: librdkafka style options, either a `dict` or a file path

        """
        opts = {**self._options, **kwargs}
        return streaming.open(*args, **opts)



#!/usr/bin/env python

__author__ = "Patrick Godwin (patrick.godwin@psu.edu)"
__description__ = "tools to parse and publish GCNs"


import argparse
import email
import os

from .io import Stream
from .models import GCNCircular, VOEvent


def read_parse_gcn(gcn_file):
    """Reads and parses a GCN circular file.

    The parsed GCN circular is formatted as a dictionary with
    the following schema:

        {'headers': {'title': ..., 'number': ..., ...}, 'body': ...}

    Args:
      gcn_file: the path to an RFC 822 formatted GCN circular

    Returns:
        the parsed GCN circular

    """
    with open(gcn_file, "r") as f:
        msg = email.message_from_file(f)

    # format gcn circular into header/body
    return GCNCircular(
        header={title.lower(): content for title, content in msg.items()},
        body=msg.get_payload(),
    )


# ------------------------------------------------
# -- main


def _add_parser_args(parser):
    parser.add_argument(
        "url",
        metavar="URL",
        help="Sets the URL (kafka://host[:port]/topic) to publish GCNs to.",
    )
    parser.add_argument(
        "gcn", metavar="GCN", nargs="+", help="One or more GCNs to publish.",
    )

    # configuration options
    config = parser.add_mutually_exclusive_group()
    config.add_argument(
        "-F", "--config-file", help="Set client configuration from file.",
    )
    config.add_argument(
        "-X",
        "--config",
        action="append",
        help="Set client configuration via prop=val. Can be specified multiple times.",
    )


def _main(args=None):
    """Parse and publish GCNs.

    """
    if not args:
        parser = argparse.ArgumentParser()
        _add_parser_args(parser)
        args = parser.parse_args()

    # load config if specified
    if args.config_file:
        config = args.config_file
    elif args.config:
        config = {opt[0]: opt[1] for opt in (kv.split("=") for kv in args.config)}
    else:
        config = None

    stream = Stream(format="json", config=config)
    with stream.open(args.url, "w") as s:
        for gcn in args.gcn:
            extension = os.path.splitext(gcn)[1]

            # parse based on GCN type
            if extension == ".gcn3":
                gcn = read_parse_gcn(gcn)
            elif extension == ".xml":
                with open(gcn, "rb") as f:
                    gcn = VOEvent.from_xml(f)
            else:
                raise ValueError(f"File extension {extension} not recognized")

            # publish GCN
            s.write(gcn.asdict())



#!/usr/bin/env python

__author__ = "Patrick Godwin (patrick.godwin@psu.edu)"
__description__ = "a module that tests the publish utilities"


from unittest.mock import patch, mock_open

from scimma.client import publish


def test_read_parse_gcn(circular_text, circular_msg):
    with patch("builtins.open", mock_open(read_data=circular_text)) as mock_file:
        gcn_file = "example.gcn3"
        gcn = publish.read_parse_gcn(gcn_file)

        # verify GCN was read in
        assert open(gcn_file).read() == circular_text
        mock_file.assert_called_with(gcn_file)

        # verify parsed GCN structure is correct
        assert gcn.header["title"] == circular_msg["header"]["title"]
        assert gcn.header["number"] == circular_msg["header"]["number"]
        assert gcn.header["subject"] == circular_msg["header"]["subject"]
        assert gcn.header["date"] == circular_msg["header"]["date"]
        assert gcn.header["from"] == circular_msg["header"]["from"]

        assert gcn.body == circular_msg["body"]



#!/usr/bin/env python

__author__ = "Patrick Godwin (patrick.godwin@psu.edu)"

import argparse
import signal

from . import __version__
from . import publish
from . import subscribe


def append_subparser(subparser, cmd, func):
    assert func.__doc__, "empty docstring: {}".format(func)
    help_ = func.__doc__.split("\n")[0].lower().strip(".")
    desc = func.__doc__.strip()

    parser = subparser.add_parser(
        cmd,
        formatter_class=argparse.RawDescriptionHelpFormatter,
        help=help_,
        description=desc,
    )

    parser.set_defaults(func=func)
    return parser


def set_up_cli():
    """Set up CLI commands for scimma entry point.

    """
    parser = argparse.ArgumentParser(prog="scimma")
    parser.add_argument(
        "--version", action="version", version=f"%(prog)s-client version {__version__}",
    )

    # set up subparser
    subparser = parser.add_subparsers(title="Commands", metavar="<command>", dest="cmd")
    subparser.required = True

    # register commands
    p = append_subparser(subparser, "publish", publish._main)
    publish._add_parser_args(p)

    p = append_subparser(subparser, "subscribe", subscribe._main)
    subscribe._add_parser_args(p)

    return parser


# ------------------------------------------------
# -- main


def main():
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    parser = set_up_cli()
    args = parser.parse_args()
    args.func(args)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

__author__ = "Bryce Cousins (bfc5288@psu.edu)"
__description__ = "a module that tests subscribe utilities"

import io
import codecs
from contextlib import redirect_stdout
from pathlib import Path

from unittest.mock import patch, mock_open

from scimma.client import subscribe


def test_print_gcn(circular_text, circular_msg):
    with patch("builtins.open", mock_open(read_data=circular_text)) as mock_file:

        # parse the sample GCN from stdout
        f = io.StringIO()
        with redirect_stdout(f):
            subscribe.print_gcn(circular_msg, json_dump=False)

        # verify circular_msg was read in
        assert open(circular_msg).read() == circular_text
        mock_file.assert_called_with(circular_msg)

        # read stdout from beginning
        f.seek(0)

        # extract GCN string from stdout
        gcn_stdout_list = f.readlines()
        gcn_stdout_str = "".join(gcn_stdout_list)

        # read in expected data txt
        shared_datadir = Path("tests/data")
        expected_raw = (
            shared_datadir / "expected_data" / "gcn_circular_stdout.txt"
        ).read_text()

        # use codec to decode the expected output to leave newline/tab characters intact
        expected = codecs.unicode_escape_decode(expected_raw)[0][:-1]  # remove ending newline char

        # verify printed GCN structure is correct
        assert gcn_stdout_str == expected



#!/usr/bin/env python

__author__ = "Patrick Godwin (patrick.godwin@psu.edu)"
__description__ = "a module that tests the io utilities"


from unittest.mock import patch, mock_open

from scimma.client import Stream


def test_stream(circular_msg, circular_text):
    with patch("scimma.client.io.streaming.open", mock_open()) as mock_stream:
        broker_url = "kafka://hostname:port/gcn"
        format = "json"
        start_at = "beginning"
        config = {}

        stream = Stream(format=format, start_at=start_at, config=config)

        # verify defaults are stored correctly
        assert stream._options["format"] == format
        assert stream._options["start_at"] == start_at
        assert stream._options["config"] == config

        with stream.open(broker_url, "w") as s:
            s.write(circular_msg)

        # verify GCN was processed
        mock_stream.assert_called_with(
            broker_url, "w", format=format, start_at=start_at, config=config
        )



#!/usr/bin/env python

__author__ = "Patrick Godwin (patrick.godwin@psu.edu)"
__description__ = "a module that tests entry points"


from unittest.mock import patch, mock_open
import sys

import pytest

from scimma.client import __version__


@pytest.mark.script_launch_mode("subprocess")
def test_cli_scimma(script_runner):
    ret = script_runner.run("scimma", "--help")
    assert ret.success

    ret = script_runner.run("scimma", "--version")
    assert ret.success

    assert ret.stdout == f"scimma-client version {__version__}\n"
    assert ret.stderr == ""


def test_cli_publish_circular(script_runner, circular_text):
    ret = script_runner.run("scimma", "publish", "--help")
    assert ret.success

    # test GCN circular
    gcn_mock = mock_open(read_data=circular_text)
    with patch("scimma.client.publish.open", gcn_mock) as mock_file, patch(
        "scimma.client.io.Stream.open", mock_open()
    ) as mock_stream:

        gcn_file = "example.gcn3"
        broker_url = "kafka://hostname:port/gcn"
        ret = script_runner.run("scimma", "publish", broker_url, gcn_file)

        # verify CLI output
        assert ret.success
        assert ret.stderr == ""

        # verify circular was processed
        mock_file.assert_called_with(gcn_file, "r")
        mock_stream.assert_called_with(broker_url, "w")


@pytest.mark.skipif(sys.version_info < (3, 7), reason="requires python3.7 or higher")
def test_cli_publish_notice(script_runner, voevent_text):
    # test GCN notice
    gcn_mock = mock_open(read_data=voevent_text.encode())
    with patch("scimma.client.publish.open", gcn_mock) as mock_file, patch(
        "scimma.client.io.Stream.open", mock_open()
    ) as mock_stream:

        gcn_file = "voevent.xml"
        broker_url = "kafka://hostname:port/gcn"
        ret = script_runner.run("scimma", "publish", broker_url, gcn_file)

        # verify CLI output
        assert ret.success
        assert ret.stderr == ""

        # verify GCN was processed
        mock_file.assert_called_with(gcn_file, "rb")
        mock_stream.assert_called_with(broker_url, "w")


def test_cli_subscribe(script_runner):
    ret = script_runner.run("scimma", "subscribe", "--help")
    assert ret.success

    with patch("scimma.client.io.Stream.open", mock_open()) as mock_stream:

        broker_url = "kafka://hostname:port/gcn"
        ret = script_runner.run("scimma", "subscribe", broker_url)

        # verify CLI output
        assert ret.success
        print(ret.stderr)
        assert ret.stderr == ""

        # verify broker url was processed
        mock_stream.assert_called_with(broker_url, "r")



import os
from setuptools import setup

# read in README
this_dir = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(this_dir, 'README.md'), 'rb') as f:
    long_description = f.read().decode().strip()

# requirements
install_requires = [
    "adc >= 0.0.2",
    "dataclasses ; python_version < '3.7'",
    "xmltodict >= 0.9.0"
]
extras_require = {
    'dev': [
        'pytest >= 5.0, < 5.4',
        'pytest-console-scripts',
        'pytest-cov',
        'flake8',
        'flake8-black',
    ],
    'docs': [
        'sphinx',
        'sphinx_rtd_theme',
        'sphinxcontrib-programoutput',
    ],
}

setup(
    name = 'scimma-client',
    description = 'A client library for SCiMMA',
    long_description = long_description,
    long_description_content_type = 'text/markdown',
    url = 'https://github.com/scimma/client_library',
    author = 'Patrick Godwin',
    author_email = 'patrick.godwin@psu.edu',
    license = 'BSD 3-Clause',

    packages = ['scimma', 'scimma.client'],
    namespace_packages = ['scimma'],

    entry_points = {
        'console_scripts': [
            'scimma = scimma.client.__main__:main',
        ],
    },

    python_requires = '>=3.6.*',
    install_requires = install_requires,
    extras_require = extras_require,
    setup_requires = ['setuptools_scm'],
    use_scm_version = {
        'write_to': 'scimma/client/_version.py'
    },

    classifiers = [
        'Development Status :: 2 - Pre-Alpha',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Astronomy',
        'Topic :: Scientific/Engineering :: Physics',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Operating System :: MacOS',
        'License :: OSI Approved :: BSD License',
    ],

)



#!/usr/bin/env python

__author__ = "Patrick Godwin (patrick.godwin@psu.edu)"
__description__ = "a module to define common message types"


from dataclasses import asdict, dataclass, field

import xmltodict


@dataclass
class VOEvent(object):
    """Defines a VOEvent 2.0 structure.

    Implements the schema defined by:
        http://www.ivoa.net/Documents/VOEvent/20110711/

    """

    ivorn: str
    role: str = "observation"
    version: str = "2.0"

    Who: dict = field(default_factory=dict)
    What: dict = field(default_factory=dict)
    WhereWhen: dict = field(default_factory=dict)
    How: dict = field(default_factory=dict)
    Why: dict = field(default_factory=dict)
    Citations: dict = field(default_factory=dict)
    Description: dict = field(default_factory=dict)
    Reference: dict = field(default_factory=dict)

    def asdict(self):
        """Represents the VOEvent as a dictionary.

        Returns:
            dict: the dict representation of the VOEvent.

        """
        return asdict(self)

    @classmethod
    def from_xml(cls, xml_input):
        """Create a new VOEvent from an XML-formatted VOEvent.

        Args:
            xml_input: a file object, string, or generator

        Returns:
            The VOEvent.

        """
        vo = xmltodict.parse(xml_input, attr_prefix="")

        # enter root and remove XML-specific namespaces
        return cls(**{k: v for k, v in vo["voe:VOEvent"].items() if ":" not in k})


@dataclass
class GCNCircular(object):
    """Defines a GCN Circular structure.

    """

    header: dict
    body: str

    def asdict(self):
        """Represents the GCN Circular as a dictionary.

        Returns:
            dict: the dict representation of the Circular.

        """
        return asdict(self)



#!/usr/bin/env python

__author__ = "Patrick Godwin (patrick.godwin@psu.edu)"
__description__ = "a module that tests models"


from scimma.client import models


def test_voevent(voevent_fileobj):
    voevent = models.VOEvent.from_xml(voevent_fileobj)

    # check a few attributes
    assert voevent.ivorn == "ivo://gwnet/LVC#S200302c-1-Preliminary"
    assert voevent.role == "observation"
    assert voevent.version == "2.0"

    assert voevent.Who["Date"] == "2020-03-02T02:00:09"
    assert voevent.Description == "Report of a candidate gravitational wave event"
    assert (
        voevent.WhereWhen["ObsDataLocation"]["ObservatoryLocation"]["id"]
        == "LIGO Virgo"
    )
